==> open-xchange-documents-backend_7.8.2.orig.tar.bz2: open-xchange-documents-backend-7.8.2/com.openexchange.office.odf/src/com/openexchange/office/odt/dom/components/CellComponent.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 *
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.odt.dom.components;

import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.json.JSONException;
import org.json.JSONObject;
import org.odftoolkit.odfdom.component.OdfOperationDocument;
import org.odftoolkit.odfdom.dom.element.OdfStylableElement;
import org.odftoolkit.odfdom.dom.element.table.TableTableCellElement;
import org.odftoolkit.odfdom.pkg.OdfFileDom;
import org.xml.sax.SAXException;

import com.openexchange.office.odf.DLList;
import com.openexchange.office.odf.DLNode;
import com.openexchange.office.odf.INodeAccessor;
import com.openexchange.office.odf.Namespaces;
import com.openexchange.office.odf.Styles;
import com.openexchange.office.odt.dom.Cell;
import com.openexchange.office.odt.dom.Paragraph;
import com.openexchange.office.odt.dom.Table;

public class CellComponent extends Component {

	final Cell cell;
	final int gridPosition;

	public CellComponent(ComponentContext parentContext, DLNode<Object> tableNode, int componentNumber, int gridPosition) {
		super(parentContext, tableNode, componentNumber);
		this.cell = (Cell)getObject();
		this.gridPosition = gridPosition;
	}

	public int getGridPosition() {
		return gridPosition;
	}

	public int getNextGridPosition() {
		return gridPosition + cell.getColumnSpan();
	}

	public Cell getCell() {
		return cell;
	}

	@Override
	public Component getNextChildComponent(ComponentContext previousChildContext, Component previousChildComponent) {

        final int nextComponentNumber = previousChildComponent != null ? previousChildComponent.getNextComponentNumber() : 0;
        DLNode<Object> nextNode = previousChildContext != null ? previousChildContext.getNode().next : ((Cell)getObject()).getContent().getFirstNode();
		while(nextNode!=null) {
			if(nextNode.getObject() instanceof Paragraph) {
				return new ParagraphComponent(this, nextNode, nextComponentNumber);
			}
			else if(nextNode.getObject() instanceof Table) {
				return new TableComponent(this, nextNode, nextComponentNumber);
			}
			nextNode = nextNode.next;
		}
		return null;
	}

    @Override
    public Component insertChildComponent(ComponentContext parentContext, DLNode<Object> contextNode, int number, Component child, Type type) {

    	DLList<Object> indexedNodeList = ((INodeAccessor)contextNode.getObject()).getContent();
        DLNode<Object> referenceNode = child != null && child.getComponentNumber()== number ? child.getNode() : null;

        switch(type) {
            case PARAGRAPH : {
            	final DLNode<Object> newParagraphNode = new DLNode<Object>(new Paragraph(null));
                indexedNodeList.addNode(referenceNode, newParagraphNode, true);
                return new ParagraphComponent(parentContext, newParagraphNode, number);
            }
            case TABLE : {
            	final DLNode<Object> newTableNode = new DLNode<Object>(new Table());
                indexedNodeList.addNode(referenceNode, newTableNode, true);
                return new TableComponent(parentContext, newTableNode, number);
            }
            default : {
                throw new UnsupportedOperationException();
            }
        }
    }

	@Override
	public void applyAttrsFromJSON(OdfOperationDocument operationDocument, JSONObject attrs)
		throws JSONException, SAXException {

		final JSONObject cellProperties = attrs.optJSONObject("cell");
		if(cellProperties!=null) {

			// TODO: big fake ... to be able to use the old "addStyle" functionality
    		// we temporarily create a TableTableCellElement to provide the actual style
    		final OdfFileDom ownerDocument = operationDocument.getOwnerDocument();
    		final OdfStylableElement e = new TableTableCellElement(ownerDocument);
    		e.setAttributeNS(Namespaces.TABLE, "table:style-name", cell.getStyleName());
    		Styles.addStyle(attrs, e, ownerDocument);
    		cell.setStyleName(e.getAttributeNS(Namespaces.TABLE, "style-name"));

			final Object gridSpan = cellProperties.opt("gridSpan");
			if(gridSpan instanceof Integer) {
				cell.setColumnSpan((Integer)gridSpan);
			}
		}
	}

	@Override
	public JSONObject createJSONAttrs(OdfOperationDocument operationDocument, JSONObject attrs)
		throws JSONException, SAXException {

		final JSONObject cellProperties = new JSONObject();

		// Hard formatted properties (automatic styles)
        final String styleName = cell.getStyleName();
        if(styleName!=null) {
	        final Map<String, Object> hardFormatting =
	        	Styles.getAutomaticStyleHierarchyProps(styleName, "table-cell", ((Styles)operationDocument.getDocument().getStylesDom()).getStyles(), attrs.asMap());
	        final Object hardCellProperties = hardFormatting.get("cell");
	        if(hardCellProperties instanceof JSONObject){
	        	final Iterator<Entry<String, Object>> hardCellPropertiesIter = ((JSONObject)hardCellProperties).entrySet().iterator();
	        	while(hardCellPropertiesIter.hasNext()) {
	        		final Entry<String, Object> hardCellEntry = hardCellPropertiesIter.next();
	        		cellProperties.put(hardCellEntry.getKey(), hardCellEntry.getValue());
	        	}
	        }
        }
		final int columnSpan = cell.getColumnSpan();
		if(columnSpan>1) {
			cellProperties.put("gridSpan", columnSpan);
		}
		if(!cellProperties.isEmpty()) {
			attrs.put("cell", cellProperties);
		}
		return attrs;
	}
}

==> open-xchange-documents-backend_7.8.3.orig.tar.bz2: open-xchange-documents-backend-7.8.3/com.openexchange.office.odf/src/com/openexchange/office/odt/dom/components/CellComponent.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 *
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.odt.dom.components;

import java.util.HashMap;

import org.json.JSONException;
import org.json.JSONObject;
import org.odftoolkit.odfdom.component.OdfOperationDocument;
import org.xml.sax.SAXException;

import com.openexchange.office.odf.DLList;
import com.openexchange.office.odf.DLNode;
import com.openexchange.office.odf.INodeAccessor;
import com.openexchange.office.odf.OpAttrs;
import com.openexchange.office.odt.dom.Cell;
import com.openexchange.office.odt.dom.Paragraph;
import com.openexchange.office.odt.dom.Table;

public class CellComponent extends Component {

	final Cell cell;
	final int gridPosition;

	public CellComponent(ComponentContext parentContext, DLNode<Object> tableNode, int componentNumber, int gridPosition) {
		super(parentContext, tableNode, componentNumber);
		this.cell = (Cell)getObject();
		this.gridPosition = gridPosition;
	}

	public int getGridPosition() {
		return gridPosition;
	}

	public int getNextGridPosition() {
		return gridPosition + cell.getColumnSpan();
	}

	public Cell getCell() {
		return cell;
	}

	@Override
	public Component getNextChildComponent(ComponentContext previousChildContext, Component previousChildComponent) {

        final int nextComponentNumber = previousChildComponent != null ? previousChildComponent.getNextComponentNumber() : 0;
        DLNode<Object> nextNode = previousChildContext != null ? previousChildContext.getNode().next : ((Cell)getObject()).getContent().getFirstNode();
		while(nextNode!=null) {
			if(nextNode.getObject() instanceof Paragraph) {
				return new ParagraphComponent(this, nextNode, nextComponentNumber);
			}
			else if(nextNode.getObject() instanceof Table) {
				return new TableComponent(this, nextNode, nextComponentNumber);
			}
			nextNode = nextNode.next;
		}
		return null;
	}

    @Override
    public Component insertChildComponent(ComponentContext parentContext, DLNode<Object> contextNode, int number, Component child, Type type) {

    	DLList<Object> indexedNodeList = ((INodeAccessor)contextNode.getObject()).getContent();
        DLNode<Object> referenceNode = child != null && child.getComponentNumber()== number ? child.getNode() : null;

        switch(type) {
            case PARAGRAPH : {
            	final DLNode<Object> newParagraphNode = new DLNode<Object>(new Paragraph(null));
                indexedNodeList.addNode(referenceNode, newParagraphNode, true);
                return new ParagraphComponent(parentContext, newParagraphNode, number);
            }
            case TABLE : {
            	final DLNode<Object> newTableNode = new DLNode<Object>(new Table());
                indexedNodeList.addNode(referenceNode, newTableNode, true);
                return new TableComponent(parentContext, newTableNode, number);
            }
            default : {
                throw new UnsupportedOperationException();
            }
        }
    }

	@Override
	public void applyAttrsFromJSON(OdfOperationDocument operationDocument, JSONObject attrs)
		throws JSONException, SAXException {

		cell.setStyleName(operationDocument.getPackage().getStyleManager().applyAttributes("table-cell", cell.getStyleName(), isContentAutoStyle(), attrs));

		final JSONObject cellProperties = attrs.optJSONObject("cell");
		if(cellProperties!=null) {
			final Object gridSpan = cellProperties.opt("gridSpan");
			if(gridSpan instanceof Integer) {
				cell.setColumnSpan((Integer)gridSpan);
			}
		}
	}

	@Override
	public void createAttrs(OdfOperationDocument operationDocument, OpAttrs attrs)
		throws SAXException {

		if(cell.getStyleName()!=null&&!cell.getStyleName().isEmpty()) {
			final OpAttrs hardCellAttrs = new OpAttrs();
			operationDocument.getPackage().getStyleManager().
				createAutoStyleAttributes(hardCellAttrs, cell.getStyleName(), "table-cell", isContentAutoStyle());
			final HashMap<String, Object> cellAttrs = hardCellAttrs.getMap("cell", false);
			if(cellAttrs!=null) {
				attrs.put("cell", cellAttrs);
			}
		}
		final int columnSpan = cell.getColumnSpan();
		if(columnSpan>1) {
			attrs.getMap("cell", true).put("gridSpan", columnSpan);
		}
	}
}
